"""
NekoBot API Wrapper
~~~~~~~~~~~~~~~~~~~
A basic wrapper for the NekoBot API.
:copyright: (c) 2022-present Bur-ham
:license: MIT, see LICENSE for more details.
"""

from typing import Literal, NamedTuple

from .api import *
from .errors import *

__title__ = 'neko'
__author__ = 'Bur-ham'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022-present Bur-ham'
__version__ = '1.0.1'


class Version(NamedTuple):
    major: int
    minor: int
    micro: int
    release: Literal['alpha', 'beta', 'stable', 'final']

version = Version(1, 0, 1, 'final')





class NekoException(Exception):
    """Base class for all neko.py exceptions."""
    pass

class NekoClientException(NekoException):
    """Base class for all neko.py client exceptions."""
    def __init__(self, message: str) -> None:
        self.message = message
        super().__init__(message)

class NekoAPIException(NekoException):
    """Base class for all neko.py API exceptions."""
    def __init__(self, message: str) -> None:
        self.message = message
        super().__init__(message)


from contextlib import AbstractAsyncContextManager
from typing import List, Literal, Optional

import aiohttp

from .client import _HTTPClient
from .models import Image

Possible = Literal['threats', 'baguette', 'clyde', 'ship', 'captcha', 'whowouldwin',
                               'changemymind', 'ddlc', 'jpeg', 'lolice', 'kannagen',
                               'iphonex', 'kms', 'animeface', 'awooify', 'trap', 'nichijou',
                               'trumptweet', 'tweet', 'kidnap', 'deepfry', 'blurpify', 'phcomment',
                               'magik', 'osu', 'clickforhentai', 'fact', 'trash', 'stickbug']

PossibleImage = Literal['hass', 'hmidriff', 'pgif', '4k', 'hentai', 
                        'holo', 'hneko', 'neko', 'hkitsune', 'kemonomimi', 'anal', 
                        'hanal', 'gonewild', 'kanna', 'ass', 'pussy', 'thigh', 'hthigh', 
                        'gah', 'coffee', 'food', 'paizuri', 'tentacle', 'boobs', 'hboobs',
                         'yaoi']


class NekoClient(AbstractAsyncContextManager):
    BASE = "https://nekobot.xyz/api"

    POSSIBLE_IMAGEGEN_ENDPOINTS = [
        'threats', 'baguette', 'clyde', 'ship', 'captcha', 'whowouldwin',
        'changemymind', 'ddlc', 'jpeg', 'lolice', 'kannagen',
        'iphonex', 'kms', 'animeface', 'awooify', 'trap', 'nichijou',
        'trumptweet', 'tweet', 'kidnap', 'deepfry', 'blurpify', 'phcomment',
        'magik', 'osu', 'clickforhentai', 'fact', 'trash', 'stickbug'
    ]

    POSSIBLE_IMAGE_ENDPOINTS = ['hass', 'hmidriff', 'pgif', '4k', 'hentai', 
                                'holo', 'hneko', 'neko', 'hkitsune', 'kemonomimi', 'anal', 
                                'hanal', 'gonewild', 'kanna', 'ass', 'pussy', 'thigh', 'hthigh', 
                                'gah', 'coffee', 'food', 'paizuri', 'tentacle', 'boobs', 'hboobs',
                                'yaoi']

    def __init__(self, session: Optional[aiohttp.ClientSession] = None) -> None:
        self._client = _HTTPClient(session)

    async def generate_image(self, endpoint: Possible, url: str, **kwargs) -> Image:
        if endpoint not in self.all_imagegen_endpoints:
            raise ValueError(f"{endpoint} is not a valid endpoint")
        url = f"{self.BASE}/imagegen?type={endpoint}&url={url}"
        response = await self._client.get(url, **kwargs)
        json_data = await response.json()
        return Image(json_data['message'])
    
    async def get_image(self, endpoint: PossibleImage) -> Image:
        if endpoint not in self.all_image_endpoints:
            raise ValueError(f"{endpoint} is not a valid endpoint")
        url = f"{self.BASE}/image?type={endpoint}"
        response = await self._client.get(url)
        json_data = await response.json()
        return Image(json_data['message'])

    @property
    def all_imagegen_endpoints(self) -> List[str]:
        return self.POSSIBLE_IMAGEGEN_ENDPOINTS

    @property
    def all_image_endpoints(self) -> List[str]:
        return self.POSSIBLE_IMAGE_ENDPOINTS
    

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> 'NekoClient':
        if not self._client.session:
            self._client.session = aiohttp.ClientSession()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self._client.close()



from typing import Optional

import aiohttp

class _HTTPClient:
    def __init__(
        self, 
        session: Optional[aiohttp.ClientSession] = None
    ) -> None:
        self.session = session

    async def get(self, url: str, **params) -> aiohttp.ClientResponse:
        if not self.session or self.session.closed:
            self.session = aiohttp.ClientSession()
        return await self.session.get(url, **params)

    async def close(self) -> None:
        if self.session:
            await self.session.close()



from distutils.core import setup
from setuptools import find_packages
import os

# User-friendly description from README.md
current_directory = os.path.dirname(os.path.abspath(__file__))
try:
    with open(os.path.join(current_directory, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()
except Exception:
    long_description = ''

setup(
	name='neko.py',
	packages=find_packages('.'),
	version='1.0.1',
	license='MIT',
	description='An API Wrapper for the NekoBot API', 
	long_description=long_description,
	long_description_content_type='text/markdown',
	author='Burham',
	author_email='',
	url='https://github.com/Bur-ham/neko.py',
	download_url='',
	keywords=['neko', 'nekobot', 'neko.py', 'nekobot.py', 'neko.py'],
	install_requires=['aiohttp'],
	classifiers=[
		'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'Typing :: Typed',
	]
)
